import { useState, isValidElement, ReactNode, MouseEvent } from "react";
import {
  Button,
  Tooltip,
  Menu,
  MenuItem,
  ListItemText,
  ListItemIcon,
  Box
} from "@mui/material";
import { AddBox, KeyboardArrowDown } from "@mui/icons-material";
import { useUniqueKeys } from "../../../hooks/useUniqueKeys";

interface MenuItem {
  text: string | ReactNode;
  action?: (
    event: MouseEvent<HTMLLIElement | HTMLAnchorElement | HTMLButtonElement>
  ) => void;
  icon?: ReactNode;
}

export interface IconMenuButtonProps {
  menuItems?: MenuItem[];
  icon?: ReactNode;
  id?: string;
  className?: string;
}

export const IconMenuButton = ({
  menuItems,
  icon = <AddBox />,
  id,
  className
}: IconMenuButtonProps) => {
  const [anchorEl, setAnchorEl] = useState<
    HTMLAnchorElement | HTMLLIElement | HTMLButtonElement | null
  >(null);
  const menuItemsKeys = useUniqueKeys(menuItems || []);

  /*
   Check if menuItems prop is an array of one
   Single button without menu must be an array of one object
  */
  const singleItem = menuItems?.length === 1 && id !== "user-login-profile-";

  const handleClick = (
    event: MouseEvent<HTMLAnchorElement | HTMLLIElement | HTMLButtonElement>
  ) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Tooltip
        title={singleItem ? menuItems[0]?.text : ""}
        arrow
      >
        <Button
          className={className}
          id={`${id}icon-menu-button-button`}
          onClick={singleItem ? menuItems[0]?.action : handleClick}
          sx={{ color: (theme) => theme.palette.primaryContrastText.main }}
          aria-label={
            // eslint-disable-next-line no-nested-ternary
            singleItem
              ? typeof menuItems[0]?.text === "string"
                ? menuItems[0]?.text
                : ""
              : undefined
          }
          aria-controls={anchorEl ? `${id}icon-menu-button-menu` : undefined}
          aria-expanded={
            // eslint-disable-next-line no-nested-ternary
            !singleItem ? (anchorEl ? "true" : "false") : undefined
          }
          aria-haspopup={singleItem ? undefined : true}
        >
          <Box
            sx={{
              mr: singleItem ? undefined : 1,
              maxHeight: (theme) => theme.spacing(3)
            }}
          >
            {icon}
          </Box>
          {!singleItem && <KeyboardArrowDown />}
        </Button>
      </Tooltip>
      {!singleItem && (
        <Menu
          id={`${id}icon-menu-button-menu`}
          anchorEl={anchorEl}
          keepMounted
          open={Boolean(anchorEl)}
          onClose={handleClose}
          MenuListProps={{ "aria-labelledby": `${id}icon-menu-button-button` }}
        >
          {menuItemsKeys &&
            menuItems?.map(({ text, action, icon }, i) => (
              <MenuItem
                key={menuItemsKeys[i]}
                disabled={!action}
                onClick={(e) => {
                  if (action) {
                    handleClose();
                    action(e);
                  }
                }}
                sx={action ? undefined : { opacity: "1.0!important" }}
              >
                {icon && <ListItemIcon>{icon}</ListItemIcon>}
                {isValidElement(text) ? (
                  text
                ) : (
                  <ListItemText
                    primary={text && text}
                    primaryTypographyProps={{ variant: "button" }}
                  />
                )}
              </MenuItem>
            ))}
        </Menu>
      )}
    </>
  );
};
